// ----------------------------------------------------------------------------
// -- Time Manager
// -- Copyright (c) 2009 by Stefan Schwope
// -- 
// -- This program is free software; you can redistribute it and/or modify it
// -- under the terms of the GNU General Public License as published by the
// -- Free Software Foundation; either version 3 of the License, or (at your
// -- option) any later version.
// --
// -- This program is distributed in the hope that it will be useful, but
// -- WITHOUT ANY WARRANTY; without even the implied warranty of
// -- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
// -- General Public License for more details.
// --
// -- You should have received a copy of the GNU General Public License along
// -- with this program; if not, see <http://www.gnu.org/licenses/>.
// --
// ----------------------------------------------------------------------------

#include <stdio.h>
#include <string.h>
#include <time.h>

// ----------------------------------------------------------------------------
#define VERSION "V1.0"
#define MAX_STR_LEN 4096

// ----------------------------------------------------------------------------
enum timeState {newDay, endOfDay, duringDay};

// ----------------------------------------------------------------------------
time_t        now;
struct tm*    ts;
FILE*         f = NULL;
char          name     [MAX_STR_LEN] = "you";
char          curTime  [MAX_STR_LEN] = "";
char          curDate  [MAX_STR_LEN] = "";
char          startTime[MAX_STR_LEN] = "";
char          startDate[MAX_STR_LEN] = "";
char          endTime  [MAX_STR_LEN] = "";
char          endDate  [MAX_STR_LEN] = "";

// ----------------------------------------------------------------------------
void logInit()
{
    FILE *fLog = fopen("stat.txt", "w");
    if(fLog != NULL)
        fclose(fLog);
}

// ----------------------------------------------------------------------------
void log(time_t start, time_t end)
{
    static unsigned long timeMonth = 0;
    static unsigned long daysMonth = 0;
    static int           curMonth  = -1;
    static int           curDay    = -1;
    static int           curYear   = -1;

    struct tm ts = *localtime(&start),
              te = *localtime(&end);
    
    if(curMonth != ts.tm_mon)
    {
        // -- write to log file -----------------
        if(curMonth != -1)
        {
            FILE *fLog = fopen("stat.txt", "a");
            if(fLog != NULL)
            {
                fprintf(fLog, "--------------------------------------------------\n");
                switch(curMonth)
                {
                    case  0: fprintf(fLog, "January %i\n",   curYear); break;
                    case  1: fprintf(fLog, "February %i\n",  curYear); break;
                    case  2: fprintf(fLog, "March %i\n",     curYear); break;
                    case  3: fprintf(fLog, "April %i\n",     curYear); break;
                    case  4: fprintf(fLog, "May %i\n",       curYear); break;
                    case  5: fprintf(fLog, "June %i\n",      curYear); break;
                    case  6: fprintf(fLog, "July %i\n",      curYear); break;
                    case  7: fprintf(fLog, "August %i\n",    curYear); break;
                    case  8: fprintf(fLog, "September %i\n", curYear); break;
                    case  9: fprintf(fLog, "October %i\n",   curYear); break;
                    case 10: fprintf(fLog, "November %i\n",  curYear); break;
                    case 11: fprintf(fLog, "December %i\n",  curYear); break;
                }

                if(daysMonth == 0)
                    daysMonth = 1;
                fprintf(fLog, "Days: %i\n",                (int) daysMonth);
                fprintf(fLog, "Time: %.2fh (%.2fh/day)\n", ((float) timeMonth)/60.0f, (((float) timeMonth)/60.0f)/daysMonth);
                fclose(fLog);
            }
        }

        // -- reset month -----------------------
        curMonth  = ts.tm_mon;
        curYear   = 1900+ts.tm_year;
        timeMonth = 0;
        daysMonth = 0;
    }

    timeMonth += (unsigned long) (difftime(end, start)/60.0);
    daysMonth++;
}

// ----------------------------------------------------------------------------
bool readLineTimeManagerFile(char *s)
{
    if(f != NULL)
    {
        if(fgets(s, MAX_STR_LEN, f) != NULL)
        {
            int len = strlen(s);
            if(len > 1)
            {
                // -- delete new line
                s[len-1] = '\0';
            } 

            return true;
        }
        else
            return false;
    }
    else
        return false;
}

// ----------------------------------------------------------------------------
time_t convertDateTime(char *sDate, char *sTime)
{
    int       yy, mm, dd, hour, min, sec; 
    struct tm when;
    long      tme;

    sscanf(sDate, "%d/%d/%d", &mm, &dd, &yy);
    sscanf(sTime, "%d:%d:%d", &hour, &min, &sec);

    if(yy < 70)
        yy+= 100; // -- year 2000 compliant
    time(&tme);
    when = *localtime(&tme);
    when.tm_year = yy;
    when.tm_mon  = mm-1;
    when.tm_mday = dd;
    when.tm_hour = hour;
    when.tm_min  = min;
    when.tm_sec  = sec;
    
    return mktime(&when);
}

// ----------------------------------------------------------------------------
void analyseTimeManagerFile()
{
    if(f == NULL)
        return;

    // -- analyse file --------------------------
    char s[MAX_STR_LEN];
    while(readLineTimeManagerFile(s))
    {
        if(strcmp(s, "#NAME:") == 0)
        {
            strcpy(name, "");
            readLineTimeManagerFile(name);
        }
        else
        if(strcmp(s, "#START:") == 0)
        {
            strcpy(startDate, "");
            strcpy(startTime, "");
            strcpy(endDate,   "");
            strcpy(endTime,   "");
            readLineTimeManagerFile(startDate);
            readLineTimeManagerFile(startTime);
        }
        else
        if(strcmp(s, "#COMMENT:") == 0)
        {
        }
        else
        if(strcmp(s, "#END:") == 0)
        {
            strcpy(endDate, "");
            strcpy(endTime, "");
            readLineTimeManagerFile(endDate);
            readLineTimeManagerFile(endTime);

            // -- log time ----------------------
            time_t start, end;
            start = convertDateTime(startDate, startTime);
            end   = convertDateTime(endDate,   endTime);
            log(start, end);
        }
        else
        {
            // -- unknown text
            if(s[0] != '-' && s[0] != '*')
                printf("ERROR: unknown text (%s)\n", s);
        }
    }
}

// ----------------------------------------------------------------------------
bool openTimeManagerFile(char *fName)
{
    f = fopen(fName, "r+");
    if(f == NULL)
        f = fopen(fName, "w+");

    analyseTimeManagerFile();
    return f!=NULL;
}

// ----------------------------------------------------------------------------
void closeTimeManagerFile()
{
    if(f != NULL)
        fclose(f);
}

// ----------------------------------------------------------------------------
void getCurrentTime()
{
    // -- get the current time
    now = time(NULL);
    ts  = localtime(&now);
    strftime(curDate, MAX_STR_LEN, "%x", ts);
    strftime(curTime, MAX_STR_LEN, "%X", ts);
}

// ----------------------------------------------------------------------------
timeState getCurrentState()
{
    // -- get current state
    if((strlen(startDate) > 0 && strlen(endDate) > 0) &&
       (strlen(startTime) > 0 && strlen(endTime) > 0) )
    {
        if(strcmp(endDate, curDate) != 0)
            return newDay;
        else
            return endOfDay;
    }
    else
    if((strlen(startDate) > 0 && strlen(endDate) == 0) &&
       (strlen(startTime) > 0 && strlen(endTime) == 0) )
        return duringDay;
    else
        return newDay;
}

// ----------------------------------------------------------------------------
void printVersion(int argc, char *argv[])
{
    printf("---------------------------------------\n"
           "-- Time Manager "VERSION"\n"
           "-- (%s - %s)\n"
           "-- Copyright (c) 2009 by Stefan Schwope\n"
           "---------------------------------------\n\n",
           curDate, curTime);
    
    switch(getCurrentState())
    {
        // --------------------------------------
        case newDay:
            printf("Good Morning %s!\n", name);
            printf("To add comments to this new day just call 'tm [comment]'.\n"
                   "'tm' without parameters finishes the day!\n");
            fprintf(f, "--------------------------------------------------\n");
            fprintf(f, "#START:\n");
            fprintf(f, "%s\n", curDate);
            fprintf(f, "%s\n", curTime);
            fprintf(f, "#COMMENT:\n");
            break;

        // --------------------------------------
        case duringDay:
            if(argc > 1)
            {
                fprintf(f, "* ");
                for(int i=1; i<argc; i++)
                    fprintf(f, "%s ", argv[i]);
                fprintf(f, "\n");
                printf("Comment added to day.\n", name);
            }
            else
            {
                fprintf(f, "#END:\n");
                fprintf(f, "%s\n", curDate);
                fprintf(f, "%s\n", curTime);
                printf("Day is over now %s - go home and relax!\n", name);
            }
            break;

        // --------------------------------------
        case endOfDay:
            printf("Day is already over - go home %s!\n", name);
            break;
    }
}

// ----------------------------------------------------------------------------
int main(int argc, char *argv[])
{
    logInit();
    if(openTimeManagerFile("tm.txt"))
    {
        getCurrentTime();
        printVersion(argc, argv);
        closeTimeManagerFile();
    }
    else
        printf("Can not open file 'tm.txt'!\n");

    return 0;
}
